@@ -2,6 +2,9 @@
 # Provides Routes for authentication, persistent session data
 # It can handle the formats : JSON , XML , YAML, PERL , HUMAN
 #
+# The subroutines of INTERNAL authorization methods must be named as __AUTH_SomeName
+# where SomeName is what you have write at the config.yml   e.g.     __AUTH_simple
+#
 # Quick diffences between plugin and Dancer2 core
 #
 #   main                             plugin                        
@@ -9,17 +12,20 @@
 #   DancerMethod                     $plugin->app->DancerMethod    
 #   setting('Key') or config->{Key}  $plugin->config->{key} ,  $plugin->app->{name}
 #
-#   change setting: $plugin->{app}->{config}->{SomeSetting} = 'foo';
+#   Change rw property        : $plugin->SomeProperty('new value');
+#   change setting (two ways) : $plugin->{app}->{config}->{plugins}->{WebService}->{SomeSetting}->{$_}->{Active} = 'foo';
+#                               $plugin->config->{SomeSetting}->{$_}->{Active} = 'foo';
 #
-#
-# George Mpouras , george.mpouras@yandex.com
-# 25 Sep 2016 , Athens - Greece
+# George Bouras, george.mpouras@yandex.com
+# 25 Sep 2016, Athens - Greece
 
 
 package Dancer2::Plugin::WebService;
 use	Dancer2::Plugin;
 use	Storable;
-our	$VERSION = '3.012';
+use	strict;
+use	warnings;
+our	$VERSION = '3.014';
 
 if ($^O =~ /(?i)MSWin/) {warn "Sorry windows operating system is not supported\n"; exit 1}
 
@@ -36,31 +42,30 @@ plugin_keywords qw/
 /;
 
 
-# Change rw properties later like   $plugin->SomeProperty('new value');
-has dir_root             => (is=>'ro', default=> sub{ ($_=$_[0]->{app}->{config}->{appdir}) =~s/\/*$//; if (-d $_) { $_ } else { warn "Could not define root directory\n"; exit 1 }});
-has formats              => (is=>'ro', default=> sub{{ json => 'application/json', xml => 'text/xml', yaml => 'text/x-yaml', perl => 'text/html', human => 'text/html' }});
-has formats_regex        => (is=>'ro', default=> sub{ $_=join '|', sort keys %{ $_[0]->formats }; $_ = qr/^($_)$/; $_ });
-has ClientIP             => (is=>'rw', default=> '');
-has route_name           => (is=>'rw', default=> '');
-has error                => (is=>'rw', default=> 0);
-has errormessage         => (is=>'rw', default=> 'ok');
-has data_from            => (is=>'rw', default=> '');		# string of user data
-has data_to              => (is=>'rw', default=> '');		# string of user data rebuilded
-has data_user            => (is=>'rw', default=> sub{ {} });	# hash   of user data, send to us at "data_from"
-has Authentication_method=> (is=>'rw', default=> '');
-has auth_member_of       => (is=>'rw', default=> sub{ [] });
-has auth_result          => (is=>'rw', default=> 0);
-has auth_message         => (is=>'rw', default=> '');
-has from                 => (is=>'rw', from_config=>'Default format',      default=> sub{ 'json' });
-has to                   => (is=>'rw', default=> sub{ $_[0]->from });
-has sudo                 => (is=>'ro', from_config=>'Command sudo',        default=> sub{ '/usr/bin/sudo' });
-has rm                   => (is=>'ro', from_config=>'Command rm',          default=> sub{ '/usr/bin/rm'   });
-has groups               => (is=>'ro', from_config=>'User must belong to one or more of the groups', default=> sub{ [] });
-has Session_idle_timout  => (is=>'ro', from_config=>'Session idle timout', default=> sub{ 3600 } );
-has rules                => (is=>'ro', from_config=>'Allowed hosts',default=> sub{ ['127.*', '192.168.*', '10.*', '172.16.*'] });
-has rules_compiled       => (is=>'ro', default=> sub {my $array = [@{$_[0]->rules}]; for (@{$array}) { s/([^?*]+)/\Q$1\E/g; s|\?|.|g; s|\*+|.*?|g; $_ = qr/^$_$/i } $array});
-has dir_session          => (is=>'ro', default=> sub {$_ = exists $_[0]->{app}->{config}->{plugins}->{WebService}->{'Session directory'} ? $_[0]->{app}->{config}->{plugins}->{WebService}->{'Session directory'} : $_[0]->dir_root .'/sessions'; $_ .= "/$_[0]->{app}->{config}->{appname}" if $_ !~/$_[0]->{app}->{config}->{appname}$/; $_});
-
+has dir_root            => (is=>'ro', default=> sub{ ($_=$_[0]->{app}->{config}->{appdir}) =~s/\/*$//; if (-d $_) { $_ } else { warn "Could not define root directory\n"; exit 1 }});
+has formats             => (is=>'ro', default=> sub{{ json => 'application/json', xml => 'text/xml', yaml => 'text/x-yaml', perl => 'text/html', human => 'text/html' }});
+has formats_regex       => (is=>'ro', default=> sub{ $_=join '|', sort keys %{ $_[0]->formats }; $_ = qr/^($_)$/; $_ });
+has ClientIP            => (is=>'rw', default=> '');
+has route_name          => (is=>'rw', default=> '');
+has error               => (is=>'rw', default=> 0);
+has errormessage        => (is=>'rw', default=> 'ok');
+has data_from           => (is=>'rw', default=> '');		# string of user data
+has data_to             => (is=>'rw', default=> '');		# string of user data rebuilded
+has data_user           => (is=>'rw', default=> sub{ {} });	# hash   of user data, send to us at "data_from"
+has auth_method         => (is=>'rw', default=> '');
+has auth_command        => (is=>'rw', default=> '');
+has auth_member_of      => (is=>'rw', default=> sub{ [] });
+has auth_result         => (is=>'rw', default=> 0);
+has auth_message        => (is=>'rw', default=> '');
+has sudo                => (is=>'ro', from_config=>'Command sudo',       default=> sub{'/usr/bin/sudo'});
+has rm                  => (is=>'ro', from_config=>'Command rm',         default=> sub{'/usr/bin/rm'});
+has rules               => (is=>'ro', from_config=>'Allowed hosts',      default=> sub{ ['127.*', '192.168.*', '10.*', '172.16.*'] });
+has Session_idle_timout => (is=>'ro', from_config=>'Session idle timout',default=> sub{ 3600 });
+has from                => (is=>'rw', from_config=>'Default format',     default=> sub{'json'});
+has to                  => (is=>'rw', default=> sub{ $_[0]->from });
+has rules_compiled      => (is=>'ro', default=> sub {my $array = [@{$_[0]->rules}]; for (@{$array}) { s/([^?*]+)/\Q$1\E/g; s|\?|.|g; s|\*+|.*?|g; $_ = qr/^$_$/i } $array});
+has dir_session         => (is=>'ro', default=> sub {$_ = exists $_[0]->{app}->{config}->{plugins}->{WebService}->{'Session directory'} ? $_[0]->{app}->{config}->{plugins}->{WebService}->{'Session directory'} : $_[0]->dir_root .'/sessions'; $_ .= "/$_[0]->{app}->{config}->{appname}" if $_ !~/$_[0]->{app}->{config}->{appname}$/; $_});
+has groups              => (is=>'ro', from_config=>'User must belong to one or more of the groups', default=> sub{ [] });
 
 
 
@@ -73,7 +78,7 @@ $plugin->config->{Routes}->{info}	= 'public';
 $plugin->config->{Routes}->{login}	= 'public';
 $plugin->config->{Routes}->{logout}	= 'private';
 
-# Default Dancer2 settings
+# Default settings
 $plugin->{app}->{config}->{content_type}= 'application/json';
 $plugin->{app}->{config}->{charset}	= 'utf-8'  if $plugin->{app}->{config}->{charset} eq '';
 $plugin->{app}->{config}->{encoding}	//= 'UTF-8';
@@ -90,37 +95,60 @@ $plugin->{app}->{config}->{plugins}->{WebService}->{'Session idle timout'}//= 36
 
 __MKDIR($plugin->dir_session) or die 'Could not create the session directory '.$plugin->dir_session." because $!\n";
 
-# Find the active authentication method
 (my $module_dir =__FILE__) =~s/\/(?>[^\/]+)$//;
 unless (-d $module_dir) {warn "Sorry could not define the Dancer2::Plugin::WebService installation directory\n"; exit 1}
+
 print STDOUT "Application name     : $plugin->{app}->{config}->{appname}\n";
 print STDOUT "Application version  : $plugin->{app}->{config}->{plugins}->{WebService}->{Version}\n";
-print STDOUT 'Run as user          : '. (getpwuid($>))[0] ."\n";
-print STDOUT 'Started at           : '. scalar(localtime $^T) ."\n";
+print STDOUT 'Run as user          : ', (getpwuid($>))[0] ,"\n";
+print STDOUT 'Started at           : ', scalar(localtime $^T) ,"\n";
 print STDOUT "Process identifier   : $$\n";
 print STDOUT "WebService version   : $VERSION\n";
 print STDOUT "Module dir           : $module_dir\n";
 
-	foreach (keys %{$plugin->config->{'Authentication method'}} ) {
-	next unless $plugin->config->{'Authentication method'}->{$_}->{Active} =~/(?i)yes/;
-	$plugin->config->{'Authentication method'}->{$_}->{Command} =~s/^MODULE_INSTALL_DIR/$module_dir/;
-	unless (-f $plugin->config->{'Authentication method'}->{$_}->{Command})	{ warn 'Sorry, could not found the external authorization utility : "'. $plugin->config->{'Authentication method'}->{$_}->{Command} ."\"\n"; exit 1 }
-	unless (-x $plugin->config->{'Authentication method'}->{$_}->{Command})	{ warn 'Sorry, the external authorization utility "'. $plugin->config->{'Authentication method'}->{$_}->{Command} .'" is not executable from user '.getpwuid($>) ."\n"; exit 1 }
-	$plugin->Authentication_method($_);	
-	print STDOUT "Authorization method : $_\n";
-	print STDOUT 'Authorization command: ', $plugin->config->{'Authentication method'}->{$_}->{Command} ,"\n";
+
+	# do not bother with the non active authentication methods
+	foreach (keys %{$plugin->config->{'Authentication methods'}}) {
+	delete $plugin->config->{'Authentication methods'}->{$_} unless ( exists $plugin->config->{'Authentication methods'}->{$_}->{Active} ) && ( $plugin->config->{'Authentication methods'}->{$_}->{Active} =~/(?i)y/ )
+	}
+
+	# Use the first active authentication method
+	foreach (keys %{$plugin->config->{'Authentication methods'}}) {
+	$plugin->auth_method($_);
+	unless (exists $plugin->config->{'Authentication methods'}->{$_}->{Command}) {warn "\nThe active Authentication method \"".$plugin->auth_method."\" does not know what to do\n"; exit 1}
+	delete $plugin->config->{'Authentication methods'}->{$_}->{Active};
+
+		# Internal
+		if ( $plugin->config->{'Authentication methods'}->{$_}->{Command} =~ /(?i)^INTERNAL$/ ) {
+		$plugin->config->{'Authentication methods'}->{$_}->{Command} = 'INTERNAL';
+		unless ( __PACKAGE__->can("__AUTH_$_") ) { warn 'Sorry, could not found the function "'. __PACKAGE__ ."::__AUTH_$_\" for INTERNAL authorization method \"$_\"\n"; exit 1 }
+		$plugin->auth_command( "__AUTH_$_" );
+		print STDOUT 'Authorization method : ', $plugin->auth_method  ," (INTERNAL)\n";
+		print STDOUT 'Authorization command: '. __PACKAGE__ ."::__AUTH_$_\n";		
+		}
+
+		# External script
+		else {
+		$plugin->config->{'Authentication methods'}->{$_}->{Command} =~s/^MODULE_INSTALL_DIR/$module_dir/;
+		$plugin->auth_command( $plugin->config->{'Authentication methods'}->{$_}->{Command} );
+		unless (-f $plugin->auth_command) { warn 'Sorry, could not found the external authorization utility : "'. $plugin->auth_command ."\"\n"; exit 1 }
+		unless (-x $plugin->auth_command) { warn 'Sorry, the external authorization utility "'. $plugin->auth_command .'" is not executable from user '.getpwuid($>) ."\n"; exit 1 }
+		my $command = $plugin->auth_command;
+		$plugin->auth_command( "\Q$command\E" );
+		$plugin->auth_command( $plugin->sudo .' '. $plugin->auth_command ) if $plugin->config->{'Authentication methods'}->{$_}->{'Use sudo'} =~/(?i)y/;
+		print STDOUT 'Authorization method : ', $plugin->auth_method  ,"\n";
+		print STDOUT 'Authorization command: ', $plugin->auth_command ,"\n";
+		}	
 	last
 	}
 
-	if ('' eq $plugin->Authentication_method) {
-	warn "\nCould not found any active authentication method, please check your configuration to activate at least one, and try again.\n";
-	exit 1
-	}
+
+#print STDERR "\n",  Data::Dumper::Dumper( $plugin->config->{'Authentication methods'} )  ,"\n\n";
 
 # Search for stored sessions
 $_ = $plugin->dir_session;
 print STDOUT "Session idle time out: ". $plugin->Session_idle_timout ."\n";
-print STDOUT "Ssession storage dir : $_\n";
+print STDOUT "Session storage dir  : $_\n";
 opendir __SESSIONDIR, $_ or die "Could not list session directory $_ because $!\n";
 
 	foreach my $session (grep ! /^\.+$/, readdir __SESSIONDIR) {
@@ -146,7 +174,7 @@ opendir __SESSIONDIR, $_ or die "Could not list session directory $_ because $!\
 	}
 
 closedir __SESSIONDIR;
-#print STDERR "\n",  Data::Dumper::Dumper( $plugin->from )  ,"\n\n";
+
 
 
 	# <after hook> to reset the any posted or user data
@@ -224,7 +252,7 @@ closedir __SESSIONDIR;
 		$plugin->RestReply(
 			Name			=> $plugin->app->{name},
 			Owner			=> $plugin->{app}->{config}->{plugins}->{WebService}->{Owner},
-			Os			=> eval{ $_='Posix'; if (open FILE, '/etc/issue') { ($_= <FILE>)=~s/\v*$//m; close FILE} $_ },
+			Os			=> eval{ local $_ = undef; local $/ = undef; open __F, -f '/etc/redhat-release' ? '/etc/redhat-release' : '/etc/issue'; if (fileno __F) { ($_= <__F>)=~s/\s*$//s; $_ = join ' ', split /\v/, $_ } close __F; $_ // $^O },
 			'Service uptime secs'	=> time - $^T,
 			'Server date time'	=> scalar localtime time,
 			Version			=> {
@@ -324,31 +352,54 @@ closedir __SESSIONDIR;
 	foreach ('user','password') {unless (exists $plugin->data_user->{$_}) { $plugin->error(6); $plugin->errormessage("Login failed, you did not pass the $_"); $plugin->app->halt( $plugin->RestReply ) }}
 	if ( $plugin->data_user->{user} =~ /^\s*$/ ) { $plugin->error(7); $plugin->errormessage("Login failed because the user is blank");     $plugin->app->halt( $plugin->RestReply ) }
 	if ( $plugin->data_user->{password} eq ''  ) { $plugin->error(8); $plugin->errormessage("Login failed because the password is blank"); $plugin->app->halt( $plugin->RestReply('user')  ) }
-	if ( 0 == @{$plugin->groups }              ) { $plugin->error(9); $plugin->errormessage("Login failed because the required group list is empty"); $plugin->app->halt( $plugin->RestReply('user') ) }
+	if ( 0 == @{ $plugin->groups }             ) { $plugin->error(9); $plugin->errormessage("Login failed because the required group list is empty"); $plugin->app->halt( $plugin->RestReply('user') ) }
 
 	$plugin->auth_result(0);
 	$plugin->auth_message('Unknown authentication error');
 	$plugin->auth_member_of([]);
 
-	my $user	= unpack 'H*',	$plugin->data_user->{user};
-	my $password	= unpack 'H*',	$plugin->data_user->{password};
-	my $groups	= join   ',',	@{$plugin->groups};
-	my $command	= $plugin->config->{'Authentication method'}->{$plugin->Authentication_method}->{'Command'};
-	my @output	= ();
-	$command	= "\Q$command\E $user $password \Q$groups\E";
-	$command	= $plugin->sudo ." $command" if $plugin->config->{'Authentication method'}->{$plugin->Authentication_method}->{'Use sudo'} =~/(?i)y/;
 
-	#print "arguments after pack: $user $password $groups\n";
-	#print STDERR "\n*",  $command  ,"*\n\n";
+		# Internal
+		if ( 'INTERNAL' eq $plugin->config->{'Authentication methods'}->{ $plugin->auth_method }->{Command} ) {
+
+		# We always call the internal authorization methods with the 4 arguments :
+		#
+		#   * username
+		#   * password
+		#   * groups that user should belong (as an array reference)
+		#   * configuration properties of the method as defined at the config.xml
+		#
+		# They must return the 3 items
+		#
+		#  * 1 (success) or 0 (fail)
+		#  * a message usually explain why the login failed
+		#  * Which groups of the defined the user belongs (reference)
+
+		my ($result, $message, $groups) = __PACKAGE__->can( $plugin->auth_command )->( $plugin->data_user->{user}, $plugin->data_user->{password}, $plugin->groups, $plugin->config->{'Authentication methods'}->{ $plugin->auth_method } );
+		$plugin->auth_result($result);
+		$plugin->auth_message($message);
+		$plugin->auth_member_of($groups) if $plugin->auth_result
+		}
+		
+
+		# External script
+		else {
+		my $user	= unpack 'H*', $plugin->data_user->{user};
+		my $password	= unpack 'H*', $plugin->data_user->{password};
+		my $groups	= join   ',',  @{$plugin->groups};
+		my @output	= ();
+		my $command	= $plugin->auth_command ." $user $password \Q$groups\E";
 	
-	# Execute the external authorization utility and capture its 3 lines output at @output array
-	open    SHELL, '-|', "$command 2> /dev/null" or die "Could run auth shell command \"$command\" because \"$?\"\n";
-	while (<SHELL>) { s/^\s*(.*?)\s*$/$1/; push @output, $_ }
-	close   SHELL;
+		# print STDERR "arguments after pack : $user $password $groups\ncommand              : $command\n\n";
+		# Execute the external authorization utility and capture its 3 lines output at @output array
+		open   SHELL, '-|', "$command 2> /dev/null" or die "Could run auth shell command \"$command\" because \"$?\"\n";
+		while(<SHELL>) { s/^\s*(.*?)\s*$/$1/; push @output, $_ }
+		close  SHELL;
+		$plugin->auth_result( $output[0]);
+		$plugin->auth_message($output[1]);
+		$plugin->auth_member_of( [ split /,/, $output[2] ] ) if $plugin->auth_result
+		}
 
-	$plugin->auth_result( $output[0]);
-	$plugin->auth_message($output[1]);
-	$plugin->auth_member_of( [ split /,/, $output[2] ] ) if $plugin->auth_result;
 
 		if ($plugin->auth_result) {
 		$plugin->auth_message('ok') if $plugin->auth_message eq '';
@@ -664,6 +715,85 @@ my $dir		= $plugin->dir_session.'/'.$plugin->data_user->{SessionID};
 }
 
 
+
+#   Internal authorization code : simple
+#   Receive the 4 arguments
+#
+#   * username
+#   * password
+#   * groups that user should belong (as an array reference)
+#   * configuration properties of the method as defined at the config.xml
+#
+# And return the 3 items
+#
+#  * 1 (success) or 0 (fail)
+#  * a message usually explain why the login failed
+#  * Which groups of the defined the user belongs (reference)
+#
+sub __AUTH_simple
+{
+my $user   = $_[0];
+my $pass   = $_[1];
+my $groups = $_[2];
+my $conf   = $_[3];
+my $result = 0;
+my $message= 'Authentication error';
+
+	if ( exists $conf->{Users}->{$user} )
+	{
+		if ( $conf->{Users}->{$user} eq '<any>' )
+		{
+		($result, $message) = (1, 'success using global password')
+		}
+		elsif ( $conf->{Users}->{$user} eq $pass )
+		{
+		($result, $message) = (1, 'success')
+		}
+		elsif ( exists $conf->{Users}->{'<any>'} )
+		{
+			if ( $conf->{Users}->{'<any>'} eq '<any>' )
+			{
+			($result, $message) = (1, 'correct user and global password for global user')
+			}
+			elsif ( $conf->{Users}->{'<any>'} eq $pass )
+			{
+			($result, $message) = (1, 'correct user and password for global user')
+			}
+			else
+			{
+			($result, $message) = (0, 'correct user but wrong password for global user')
+			}
+		}
+		else
+		{
+		($result, $message) = (0, 'wrong password')
+		}
+	}
+	elsif ( exists $conf->{Users}->{'<any>'} )
+	{
+		if ( $conf->{Users}->{'<any>'} eq '<any>' )
+		{
+		($result, $message) = (1, 'success for global user and global password')
+		}
+		elsif ( $conf->{Users}->{'<any>'} eq $pass )
+		{
+		($result, $message) = (1, 'success for global user and password')
+		}
+		else
+		{
+		($result, $message) = (0, 'wrong password for global user')
+		}
+	}
+	else
+	{
+	($result, $message) = (0, 'invalid user')
+	}
+
+$result, $message, $groups
+}
+
+
+
 #   Returns the posted or sent data 
 #
 #   my ($var1, $var2) = get_data_user('k1', 'k2');    # returns the selected keys
@@ -684,27 +814,19 @@ my $plugin = shift;
 
 
 #   Set new user data as if they were sent or posted
-#   It returns the data like the get_user_data
 #
-#   my %data = set_data_user(   new1 => 'foo1', new2 => 'foo2'   );   # return the keys
-#   my %data = set_data_user( { new1 => 'foo1', new2 => 'foo2' } );   # return the keys
+#   set_data_user(   new1 => 'foo1', new2 => 'foo2'   );  # or
+#   set_data_user( { new1 => 'foo1', new2 => 'foo2' } );
 #
 sub set_data_user
 {
 my $plugin = shift;
-my @keys;
+return unless @_;
+if (( 1 == @_ ) && ( 'HASH' eq ref $_[0] )) { @_ = %{ $_[0] } }
 
-	if (@_)
+	for (my($i,$j)=(0,1); $i < scalar(@_) - (scalar(@_) % 2); $i+=2,$j+=2)
 	{
-	if (( 1 == @_ ) && ( 'HASH' eq ref $_[0] )) { @_ = %{ $_[0] } }
-
-		for (my($i,$j)=(0,1); $i < scalar(@_) - (scalar(@_) % 2); $i+=2,$j+=2)
-		{
-		push @keys, $_[$i];
-		$plugin->data_user->{$_[$i]} = $_[$j]
-		}
-
-	map {$_, $plugin->data_user->{$_}} @keys
+	$plugin->data_user->{$_[$i]} = $_[$j]
 	}
 }
 
@@ -873,7 +995,6 @@ __END__
 =pod
 
 =encoding UTF-8
-=pod
 
 =head1 NAME
 
@@ -881,17 +1002,9 @@ Dancer2::Plugin::WebService - Rapid creation of RESTful Web Services with sessio
 
 =head1 VERSION
 
-version 3.012
+version 3.014
 
-=head2 NAME
-
-Dancer2::Plugin::WebService Rapid creation of RESTful Web Services with sessions and persistent data
-
-=head2 VERSION
-
-version 3.012
-
-=head2 SYNOPSIS
+=head1 SYNOPSIS
 
 Using I<curl> ( L<https://curl.haxx.se> ) as client. At your main script you can use all Dancer2 core methods.
 
@@ -958,7 +1071,15 @@ Your script (example)
 
   dance;
 
-=head2 POLYMORPHISM
+=head1 NAME
+
+Dancer2::Plugin::WebService - Rapid creation of RESTful Web Services with sessions and persistent data
+
+=head1 VERSION
+
+version 3.014
+
+=head1 POLYMORPHISM
 
 Dancer2::Plugin::WebService can handle as input or output multiple formats
 
@@ -986,7 +1107,7 @@ If missing the default is json. The "to" is the same as "from" if missing. e.g.
   curl --data '<Data><k1>3</k1><k2>30</k2></Data>' 'localhost:65535/test_get_data?from=xml;to=human'
   curl --data '<Data><k1>3</k1><k2>30</k2></Data>' 'localhost:65535/test_get_data?from=xml;to=yaml'
 
-=head2 ROUTES
+=head1 ROUTES
 
 Your routes can be either B<public> or B<private>
 
@@ -1007,12 +1128,12 @@ or at your main script
 
   setting('plugins')->{'WebService'}->{'Routes'}->{'SomeRoute'} = 'private';
 
-=head2 BUILT-IN ROUTES
+=head1 BUILT-IN ROUTES
 
 There are some built in routes for your convenience.
 You can use the "from" and "to" format modifiers if you want
 
-=head3 info/version
+=head2 info/version
 
 Service information (public route)
 
@@ -1022,17 +1143,17 @@ Service information (public route)
   curl localhost:65535/info/version?to=perl
   curl localhost:65535/info/version?to=human
 
-=head3 info/client
+=head2 info/client
 
 Client information (public route)
 
   curl localhost:65535/info/client
 
-=head3 info
+=head2 info
 
 Redirects to /info/version
 
-=head3 login
+=head2 login
 
 Login for using private routes and storing persistent data. It is a public route.
 
@@ -1053,14 +1174,14 @@ You can control which clients are allowed to login by editing the file I<config.
           - ????:????:????:6d00:20c:29ff:*:ffa3
           - "*"
 
-=head3 logout
+=head2 logout
 
 It is private route as you can not logout without login . In order to logout you must know the SessionID . If you logout you can not use the private routes and all coresponded session data are deleted.
 
   curl -X GET --data '{"SessionID":"0a1ad34505076d930c3f76c52645e54b"}'  localhost:65535/logout
   curl -X GET --data '{"SessionID":"0a8e4f0523dafa980ec35bcf29a5cc8c"}' 'localhost:65535/logout?from=json;to=xml'
 
-=head2 SESSIONS
+=head1 SESSIONS
 
 The sessions auto expired after some seconds of inactivity. You can change the amount of seconds either at the I<config.yml>
 
@@ -1078,9 +1199,9 @@ You can change Session persistent data storage directory at the I<config.yml>
     WebService:
       Session directory : /usr/local/sessions
 
-=head2 METHODS
+=head1 METHODS
 
-=head3 RestReply
+=head2 RestReply
 
 send the reply to the client. This should be the last route's statement
 
@@ -1091,49 +1212,49 @@ send the reply to the client. This should be the last route's statement
   RestReply('DATA_USER_ALL')          data send by the user with any addtions
   RestReply('k1')                     data send by the user only one key
 
-=head3 get_data_user
+=head2 get_data_user
 
 Retrieves data user send to WebService with his client e.g. curl or wget . We use this to do something usefull with user's data. This is normally the method you will use more often
 
   my ($var1, $var2) = get_data_user('k1', 'k2');    # return the selected keys
   my %hash          = get_data_user();              # return all data as hash
 
-=head3 set_data_user
+=head2 set_data_user
 
-You can define extra data. Try to use this instead of common $variables , @arrays , etc because they are visible to other users of the service !
+You can define extra data. Use this instead of common $variables , @arrays , etc because they are visible to other users of the service !
 
-  my %data = set_data_user(   new1 => 'foo1', new2 => 'foo2'   );   # return the keys
-  my %data = set_data_user( { new1 => 'foo1', new2 => 'foo2' } );   # return the keys
+  set_data_user(   new1 => 'foo1', new2 => 'foo2'   );  # or
+  set_data_user( { new1 => 'foo1', new2 => 'foo2' } );
 
-=head3 del_data_user
+=head2 del_data_user
 
 Deletes data, think it as the opposite of B<set_data_user>
 
   del_data_user( 'k1', 'k2', ... );    # delete only the selected keys
   del_data_user();                     # delete all keys
 
-=head3 set_data_session
+=head2 set_data_session
 
 Store persistent session data. Session data are not volatile like the user data between service calls. You have to login prior using this method.
 
   set_data_session(   new1 => 'foo1', new2 => 'foo2'   );
   set_data_session( { new1 => 'foo1', new2 => 'foo2' } );
 
-=head3 get_data_session
+=head2 get_data_session
 
 Retrieves session data. You have to login prior using this method.
 
   my %data = get_data_session( 'k1', 'k2', ... );    # return only the selected keys
   my %data = get_data_session();		     # returs all keys
 
-=head3 del_data_session
+=head2 del_data_session
 
 Deletes session data. You have to login prior using this method.
 
   del_data_session( 'k1', 'k2', ... );   # deletes only the selected keys
   del_data_session();                    # deletes all keys
 
-=head2 INSTALLATION
+=head1 INSTALLATION
 
 After install I<Dancer2::Plugin::WebService> create your application e.g I<TestService> using the command inside e.g. the I</opt> folder
 
@@ -1150,7 +1271,7 @@ Assuming that you will start the service as a non privileged user e.g. I<joe>
 
 If you want compressed replies edit the file I</opt/TestService/bin/app.psgi>
 
-  #!/usr/bin/perl
+  #!/usr/bin/env perl
   use FindBin;
   use lib "$FindBin::Bin/../lib";
   use TestService;
@@ -1159,7 +1280,7 @@ If you want compressed replies edit the file I</opt/TestService/bin/app.psgi>
 
 Or if you have slow CPU use uncompressed replies by editing the file I</opt/TestService/bin/app.psgi>
 
-  #!/usr/bin/perl
+  #!/usr/bin/env perl
   use FindBin;
   use lib "$FindBin::Bin/../lib";
   use TestService;	
@@ -1182,12 +1303,11 @@ I<.../environments/production.yml>
 
 Edit the file I</opt/TestService/config.yml>
 
-  appname      : TestService
-  environment  : production
-  
-  plugins:
-    WebService:
-      Version             : 1.0.0
+  appname                 : TestService
+  environment             : production
+  plugins                 :
+    WebService            :
+      Version             : 2.0.1
       Owner               : Joe Lunchbucket, Joe.Lunchbucket@example.com
       Session directory   : /usr/local/sessions
       Session idle timout : 3600
@@ -1195,10 +1315,12 @@ Edit the file I</opt/TestService/config.yml>
       Command sudo        : /usr/bin/sudo
       Command rm          : /usr/bin/rm
       Routes              :
-        test1             : public
-        test2             : public
-        test3             : private
-        
+        test_mirror       : public
+        test_get_one_key  : public
+        test_get_data     : public
+        test_new_data     : public
+        test_session      : private
+  
       Allowed hosts:
   
         - 127.*
@@ -1215,15 +1337,19 @@ Edit the file I</opt/TestService/config.yml>
         - power
         - storage
         - network
-  
-      Authentication method:
-        Always allow login for testing:
-          Command  : MODULE_INSTALL_DIR/scripts/AlwaysOk/AlwaysOk.sh
-          Active   : no
-          Use sudo : no
+
+      Authentication methods:
+        simple:
+          Command  : INTERNAL
+          Active   : yes
+          Users    :
+            <any>  : secret4all
+            user1  : <any>
+            user2  : pass2
+            user2  : pass2
         Linux native users:
           Command  : MODULE_INSTALL_DIR/scripts/LinuxOS/AuthUser.pl
-          Active   : yes
+          Active   : no
           Use sudo : yes
         Basic Apache auth for simple users:
           Command  : MODULE_INSTALL_DIR/scripts/HttpBasic/users.pl
@@ -1233,14 +1359,6 @@ Edit the file I</opt/TestService/config.yml>
           Command  : MODULE_INSTALL_DIR/scripts/HttpBasic/admins.pl
           Active   : no
           Use sudo : no
-        Active directory:
-          Command  : MODULE_INSTALL_DIR/scripts/ActiveDirectory/ActiveDirectory.pl
-          Active   : no
-          Use sudo : no
-        LDAP:
-          Command  : MODULE_INSTALL_DIR/scripts/LDAP/LDAP.pl
-          Active   : no
-          Use sudo : no
 
 Write your code at the file  I</opt/TestService/lib/TestService.pm>
 
@@ -1297,14 +1415,23 @@ or without Plack
 
 if you want to install your WebService application as Linux service please readme the INSTALL
 
-=head2 AUTHENTICATION
+=head1 AUTHENTICATION
 
-For using B<private> methods and persistent session data you have to login. B<login> is handled from external scripts/programs .
-It is very easy to write your own script to have authentication with anything you can imagine.
+For using B<private> methods and persistent session data you have to login. B<login> is handled from external scripts/programs.
 The authentication is configured at config.xml
-Only one authentication method can ne active at any time.
+Only one Authentication method can ne active at any time.
 The authentication scripts must be executable from the user running the service.
-Some scripts need elevated privileges so you need to enable sudo for them. E.g. for the native Linux mechanism we have
+
+It is very easy to write your own scripts to have any authentication you want. For writing your own scripts please read the AUTHENTICATION_SCRIPTS and review the existing scripts
+
+Some scripts need elevated privileges, so you must to enable sudo for them. You need also to add the the user ( I<joe> ) running the appplication and the coresponded script at the B</etc/sudoers> file e.g.
+
+  # This is needed for the linux user authorization of the WebService
+  joe ALL=NOPASSWD: /usr/local/share/perl5/Dancer2/Plugin/scripts/LinuxOS/AuthUser.pl
+
+If you want to use a method you must set  I<Active : yes>
+
+Lets view as an example the native Linux mechanism. We have
 
     User must belong to one or more of the groups:
 
@@ -1312,22 +1439,40 @@ Some scripts need elevated privileges so you need to enable sudo for them. E.g.
       - storage
       - network
 
-    Authentication method:
+    Authentication methods:
 
       Linux native users:
         Command  : MODULE_INSTALL_DIR/scripts/LinuxOS/AuthUser.pl
         Active   : yes
         Use sudo : yes
 
-That means that if a user do not belong to any of the groups I<power , storage , network> the login will fail . We want to use thie method so we have I<Active : yes> . Also because this work only with root priviliges we have I<Use sudo : yes>
+That means that if a user do not belong to any of the groups e.g. I<power , storage , network> the login will fail. Also because this work only with root priviliges we have I<Use sudo : yes>
 
-That means if you running the service as user joe , at the file B</etc/sudoers> must exist a line similar to
+There are also built in authentication methods that do not need external scripts.
 
-  joe ALL=NOPASSWD: /usr/share/perl5/site_perl/Dancer2/Plugin/scripts/LinuxOS/AuthUser.pl
+For example the B<simple> method define the users and their passwords directly the B<config.yml> file. It can be configured like 
 
-For writing your own script please read the scripts howto.txt and review the existing scripts
+    Authentication methods:
+      simple:
+        Command  : INTERNAL
+        Active   : yes
+          Users  :
 
-=head2 SEE ALSO
+            <any> : secret4all
+            user1 : <any>
+            user2 : pass2
+            user2 : pass2
+
+The <any> means ... any ! So if you want to use it for login no matter the username or the password you could have
+
+    Authentication methods:
+      simple:
+        Command  : INTERNAL
+        Active   : yes
+          Users  :
+            <any>: <any>
+
+=head1 SEE ALSO
 
 B<Plack::Middleware::REST> Route PSGI requests for RESTful web applications
 
@@ -1341,7 +1486,7 @@ B<XML::RPC> Pure Perl implementation for an XML-RPC client and server.
 
 B<JSON::RPC> JSON RPC 2.0 Server Implementation 
 
-=head2 COPYRIGHT AND LICENSE
+=head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2017 by George Bouras
 
@@ -1360,3 +1505,16 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
+
+=head1 AUTHOR
+
+George Bouras <george.mpouras@yandex.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2017 by George Bouras.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
